/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-build.h"

// Run the autogen.sh command if a project is currently open.
void openldev_build_auto_generate (GtkMenuItem *menuitem,
                                   gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("You cannot run Auto Generate until you open a project! You can create a new project or open one that already exists in the File menu.");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
    chdir (openldev->project->location.c_str());
    
    string command = openldev->project->autogen + " 2>&1";
    string message = "Running: " + openldev->project->autogen;
    openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());
    bool result = openldev_output_build_run_command (openldev->output->build, 
                                                     command.c_str(), openldev->env);
    
    if (result || !result)
      openldev_status_bar_unlock_message (openldev->statusbar);
  }
}

// If a project is open, build it. Run autogen.sh and then build again if
// Makefiles are not found. All of the files are first saved.
void openldev_build_project (GtkMenuItem *menuitem,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("You cannot build until you open a project! You can create a new project or open one that already exists in the File menu.");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
    openldev_menu_file_save_all (openldev);
    chdir (openldev->project->location.c_str());
    
    string command = openldev->project->build + " 2>&1";
    string message = "Running: " + openldev->project->build;
    openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());

    if (!openldev_output_build_run_command (openldev->output->build, 
                                            command.c_str(), 
                                            openldev->env))
    {
      openldev_status_bar_unlock_message (openldev->statusbar);
      openldev_menu_build_auto_generate (openldev);
      openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());
      bool result = openldev_output_build_run_command (openldev->output->build, 
                                                       command.c_str(), 
                                                       openldev->env);
      
      if (result || !result)
        openldev_status_bar_unlock_message (openldev->statusbar);
    }
    else
      openldev_status_bar_unlock_message (openldev->statusbar);
  }
}

// Clean all of the sources, build files, object files and those files
// created by configure
void openldev_build_clean (GtkMenuItem *menuitem,
                           gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("You cannot run clean until you open a project! You can create a new project or open one that already exists in the File menu.");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
    chdir (openldev->project->location.c_str());
    
    string command = openldev->project->clean + " 2>&1";
    string message = "Running: " + openldev->project->clean;
    openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());
    bool result = openldev_output_build_run_command (openldev->output->build, 
                                                     command.c_str(), openldev->env);
    
    if (result || !result)
      openldev_status_bar_unlock_message (openldev->statusbar);
  }
}

// Compile current file if a file and a project is open. The function then
// creates the compile string from the project language & runs the command
void openldev_build_compile (GtkMenuItem *menuitem,
                             gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (!openldev_file_manager_get_file_open (openldev->notebook))
    openldev_error ("You cannot compile unless there is a file open! You can create a new file or open one that already exists in the File menu.");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output->notebook), 0);
    string command = openldev->project->compile,
           fn = openldev_file_manager_get_current_filename (openldev->notebook);
    
    int loc = command.find ("{FILE}", 0);
    while (loc != string::npos)
    {
      command.erase (loc, 6);
      command.insert (loc, fn);
      loc = command.find ("{FILE}", 0);
    }
    
    string message = "Running: " + command;
    string location = openldev->project->location + "src/";
    chdir (openldev->project->location.c_str());

    command += " 2>&1";
    openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());
    bool result = openldev_output_build_run_command (openldev->output->build, 
                                                     command.c_str(), openldev->env);
    
    if (result || !result)
      openldev_status_bar_unlock_message (openldev->statusbar);
  }
}

// If a project is open, run the ./configure command in the project directory
void openldev_build_configure (GtkMenuItem *menuitem,
                               gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("You cannot run configure unless there is a project open! You can create a new file or open one that already exists in the File menu.");
  else 
  {
    string configure = openldev->project->configure;
    if (configure.find ("./configure", 0) != string::npos)
      configure.erase (configure.find ("./configure", 0), 11);
    if (configure[0] == ' ')
      configure.erase (0, 1);
    
    configure = openldev_input_dialog_run ("Parameters:", 
                                           "Configure Parameters",
                                           configure.c_str());
    
    if (configure != "EMPTY")
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output->notebook), 0);
      chdir (openldev->project->location.c_str());
      
      string command = "./configure " + configure + " 2>&1";
      openldev->project->configure = "./configure " + configure;
      openldev_project_settings_write_project_file (openldev->project);
      string message = "Running: " + openldev->project->configure;
      openldev_status_bar_lock_message (openldev->statusbar, (gchar*) message.c_str());
      bool result = openldev_output_build_run_command (openldev->output->build, 
                                                       command.c_str(), openldev->env);
    
      if (result || !result)
        openldev_status_bar_unlock_message (openldev->statusbar);
    }
  }
}

// If a project is open, run the executable in the terminal specified
// by the user 
void openldev_build_execute (GtkMenuItem *menuitem,
                             gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("You cannot run this command unless there is a project open! You can create a new file or open one that already exists in the File menu.");
  else 
  {
    string command = "gnome-terminal -e \"openldev_launcher " + openldev->project->location 
                   + "src/" + openldev->project->name + "\"";
    system (command.c_str());
  }
}

// Stop the current compilation, clean, configure, autogen or build
void openldev_build_stop (GtkMenuItem *menuitem,
                          gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  openldev_output_build_stop_build (openldev->output->build);
}
