/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-file.h"

#define NUM_MENUITEMS 17
gchar* menuitems[] = { "/MenuBar/File/FileSave", "/MenuBar/File/FileSaveAs", 
       "/MenuBar/File/FileSaveAll", "/MenuBar/File/FileRevert", "/MenuBar/File/FilePrint", 
       "/MenuBar/File/FilePrintPreview", "/MenuBar/Edit/EditUndo", "/MenuBar/Edit/EditRedo", 
       "/MenuBar/Edit/EditCut", "/MenuBar/Edit/EditCopy", "/MenuBar/Edit/EditPaste", 
       "/MenuBar/Edit/EditSelectAll", "/MenuBar/Edit/EditDeselect", "/ToolBar/FilePrint",
       "/ToolBar/FileSave", "/ToolBar/EditUndo", "/ToolBar/EditRedo" };

void openldev_on_delete_tab (GtkWidget*, gpointer);
void openldev_enable_project_menu_items (OpenLDev*);
void openldev_file_connect_signals (OpenLDev*);
void openldev_add_recent_file (OpenLDev*, const char*);
string openldev_file_get_text_in_buffer (GtkSourceBuffer*);

// Create a new file. This doesn't add the file to the project. The user must
// do that manually through the makefile editor.
void openldev_file_new_file (GtkMenuItem *item, 
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  // Only create new file if project is open
  else 
  {
    string file;
    NewFileDialog *dialog = openldev_new_file_dialog_new ((gchar*) openldev->project->location.c_str());
    openldev_new_file_dialog_run (dialog, file);
    
    if (file != "NULL" && file != "")
    {
      openldev_browser_project_refresh (openldev->browser->files);
      openldev_open_file (openldev, (gchar*) file.c_str());     
    }
    
    gtk_widget_destroy (dialog->dialog);
    delete dialog;
  }
}

// Create a new project by calling the new project dialog & then open the
// project that was created.
void openldev_file_new_project (GtkMenuItem* menuitem, 
                                gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  NewProjectDialog *dialog = openldev_new_project_dialog_new ();
  GList *list = openldev_new_project_dialog_run (dialog);

  // Open newly created project if OK is pressed
  if (list != NULL)
  {
    string project_file = (const char*) g_list_nth_data (list, 0);
    string command = (const char*) g_list_nth_data (list, 1);
    
    gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output->notebook), 0);
    gboolean result = openldev_output_build_run_command (openldev->output->build, 
                                                command.c_str(), openldev->env);

    if (result)
      if (openldev_open_project (openldev, (gchar*) project_file.c_str()))
        openldev_menu_build_auto_generate (openldev);
  }
  
  delete dialog;
}

// Get the file that needs to be opened. This function doesn't actually open the
// file. Therefore, it is tricky and lies, but oh well.
void openldev_file_open (GtkMenuItem *item, 
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;

  // Create file selector
  GtkWidget *dialog;
  dialog  = gtk_file_chooser_dialog_new ("Choose a project or File", 
                                         GTK_WINDOW(openldev->window),
                                         GTK_FILE_CHOOSER_ACTION_OPEN, 
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_OK,
                                         NULL);
  
  // Set default folder
  if (!openldev->project->name.empty())
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         openldev->project->location.c_str());
  else
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         g_get_home_dir());  
  
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .openldev file
      string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      gtk_widget_destroy (dialog);
      bool result;
      
      if (filename.find (".openldev", 0) != string::npos)
        result = openldev_open_project (openldev, (gchar*) filename.c_str());
      else
        result = openldev_open_file (openldev, (gchar*) filename.c_str());
      break;
    }
    default:
      gtk_widget_destroy (dialog);
      break;
  }
}

// Open the specified file if it exists. This function receives the filename that
// it is supposed to open. If a project is open, the file is added as the
// last opened file. The file is then opened, syntax highlighting is applied, 
// line numbering is calculated and displayed, signals are connected to the
// keystroke action and final initialization is applied
gboolean openldev_open_file (OpenLDev *openldev,
                             gchar *file)
{
  string filename = file;
  
  if (!g_file_test (filename.c_str(), G_FILE_TEST_EXISTS))
  {
    openldev_error ("Cannot open file. It does not exist!");
    
    if (!openldev->project->name.empty())
    {
      string last = "(" + filename + ")";
      string lastfiles = openldev->project->lastfiles;
      
      lastfiles.erase (lastfiles.find (last, 0), last.length());
      openldev_project_settings_set_lastfiles (openldev->project, lastfiles.c_str());
      openldev_project_settings_write_project_file (openldev->project);
    }
    
    return FALSE;
  }
  else
  {
    // Add a new tab to the notebookmanager
    if (openldev_file_manager_add_tab (openldev->notebook,
                                       filename.c_str(),
                                       openldev->env))
    { 
      string text, title, cliptext;

      // Set the file as a last opened file
      if (!openldev->project->name.empty())
      {
        string temp = "(" + filename + ")";
        if (openldev->project->lastfiles.find(temp, 0) == string::npos)
        {
          openldev_project_settings_set_lastfiles (openldev->project,
                           (openldev->project->lastfiles + temp).c_str());
          openldev_project_settings_write_project_file (openldev->project);
        }
      }
  
      // Open file & read in text
      text = openldev_read_file (openldev_file_manager_get_current_filename(openldev->notebook));
      GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
      
      // Make sure the initial population of the view is not added to undo
      gtk_source_buffer_begin_not_undoable_action (buffer);
      gtk_text_buffer_set_text (GTK_TEXT_BUFFER(buffer), text.c_str(), -1);
      gtk_source_buffer_end_not_undoable_action (buffer);
      
      // Set the text to the buffer, connect keystroke function & show/hide line numbering
      gtk_text_view_set_buffer (GTK_TEXT_VIEW (openldev_file_manager_get_textview
                                (openldev->notebook)), GTK_TEXT_BUFFER(buffer));
      
      openldev_file_manager_set_properties (openldev->notebook, FALSE, openldev->env);
      openldev_file_connect_signals (openldev);
      
      // Set the font for the new file
      gchar *font_description = g_strconcat (openldev->env->font_face.c_str(), " ",
                                             openldev->env->font_size.c_str(), NULL);
      PangoFontDescription *fd = pango_font_description_from_string (font_description);
      gtk_text_view_set_tabs (GTK_TEXT_VIEW 
               (openldev->notebook->tabs[openldev->notebook->tab_count-1]->textview), 
               openldev_file_manager_get_tab_array (fd, openldev->env, 
               openldev->notebook->tabs[openldev->notebook->tab_count-1]->textview));
      openldev_file_manager_modify_fonts (openldev->notebook, fd);
      openldev_manage_menu_items (openldev);
      openldev_add_recent_file (openldev, filename.c_str());
      g_free (font_description);
    }
  }
  
  return TRUE;
}

void openldev_add_recent_file (OpenLDev *openldev,
                               const char *filename)
{
  // Add a recent file & then repopulate them
  openldev_environment_add_recent_file (openldev->env, filename);
  for (int i=0; i < 5; i++)
  {
    string recent = "/MenuBar/File/FileRecent";
    gchar buff[2];
    sprintf (buff, "%d", i+1);
    recent += buff;
  
    if (openldev->env->recent_files[i].empty())
      gtk_widget_hide (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                       recent.c_str()));
    else
    {
      string name = string (buff) + ". " + string (openldev_environment_get_recent_short (openldev->env, i));
      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (openldev->menus->uimanager, recent.c_str())))),
                          name.c_str());
      gtk_widget_show (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                       recent.c_str()));
    }
  }
}

// Connect the signals for the newly opened file
void openldev_file_connect_signals (OpenLDev *openldev)
{
  g_signal_connect (G_OBJECT (openldev_file_manager_get_textview (openldev->notebook)), 
                    "key-press-event",
                    G_CALLBACK (openldev_on_textview_keypress), 
                    (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev_file_manager_get_textview (openldev->notebook)), 
                    "event",
                    G_CALLBACK (openldev_on_textview_event), 
                    (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->notebook->tabs[openldev->notebook->tab_count-1]->button),
                    "clicked",
                    G_CALLBACK (openldev_on_delete_tab),
                    (gpointer) openldev);
}

// The project specified is opened. This function receives the filename of the
// project file. If the project was loaded, the last file opened is reloaded,
// the filelist is populated and the last project open is set to this project
gboolean openldev_open_project (OpenLDev *openldev,
                                gchar *file)
{
  string filename = file;
  openldev_file_manager_erase_all_tabs (openldev->notebook);
  gboolean success = openldev_project_settings_load_project_file (openldev->project, 
                                                                  filename.c_str());
 
  // If the project was opened
  if (success)
  {
    // Open the last file if it exists   
    string files = openldev->project->lastfiles;
    while (!files.empty())
    {
      int l = files.find (")", 0);
      string temp = files.substr (0, l + 1);
      files.erase (0, l + 1);
      
      temp.erase (0, 1);
      temp.erase (temp.length() - 1, 1);
      openldev_open_file (openldev, (gchar*) temp.c_str());
    }

    // Make environment ready for project
    openldev_populate_treeview (openldev);
    openldev_environment_set_last_project (openldev->env, filename.c_str());
    
    // Enable all of the project menu items because a project is now open
    openldev_enable_project_menu_items (openldev);
    return TRUE;
  }
  
  return FALSE;
}

// Enable all of the project menu items because a project is now open
void openldev_enable_project_menu_items (OpenLDev *openldev)
{
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/MenuBar/File/FileClose"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/MenuBar/Project"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/MenuBar/Build"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/ToolBar/BuildCompileToolbar"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/ToolBar/BuildBuildToolbar"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/ToolBar/BuildExecute"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/ToolBar/BuildClean"), TRUE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                            "/ToolBar/BuildStop"), TRUE);
}

// Save the current file & mark the file as unmodified. If the file does not
// have a name, then save as is called
void openldev_file_save (GtkMenuItem* menuitem, 
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    string current_fn = openldev_file_manager_get_current_filename (openldev->notebook);
    if (current_fn.empty() || current_fn == "EMPTY")
      openldev_menu_file_save_as (openldev);
  
    // Save the file & set as unmodified
    GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
    openldev_write_file (openldev_file_manager_get_current_filename (openldev->notebook),
                         openldev_file_get_text_in_buffer (buffer));

    int current = gtk_notebook_get_current_page (GTK_NOTEBOOK(openldev->notebook->notebook));
    openldev_file_manager_mark_current_tab_modified (openldev->notebook, FALSE, current);
  }
}

// Get all of the text in the GtkSourceBuffer and return it.
string openldev_file_get_text_in_buffer (GtkSourceBuffer *buffer)
{
  GtkTextIter start, end;
  gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (buffer), &start);
  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (buffer), &end);
  return gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &start, &end, TRUE);
}

// Save the current file by a different name. The FileChooserDialog allows the
// user to choose the location and filename. Note: This function does not actually
// save the file, but rather just chooses a new name for it & calls on_file_save
void openldev_file_save_as (GtkMenuItem* menuitem, 
                            gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkWidget *dialog;
    dialog = gtk_file_chooser_dialog_new ("Save file as", 
                                          GTK_WINDOW(openldev->window),
                                          GTK_FILE_CHOOSER_ACTION_SAVE,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                          NULL);
  
    // Set default folder
    string folder_name = openldev->project->location;
    
    if (!openldev->project->name.empty())
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           folder_name.c_str());  
    else
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           g_get_home_dir());
  
    // Add the filename to the text entry in SaveAs dialog
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog),
          openldev_file_manager_get_current_short_filename (openldev->notebook).c_str());
  
    // Open fileselection dialog to get file location
    int result = gtk_dialog_run (GTK_DIALOG(dialog));
    switch (result)
    {
      // If OK is pressed
      case (GTK_RESPONSE_OK):
      {
        if (!openldev->project->name.empty())
        {
          string old = "(" + openldev_file_manager_get_current_filename (openldev->notebook) + ")";
          if (openldev->project->lastfiles.find (old, 0) != string::npos)
          {
            string temp = openldev->project->lastfiles;
            int loc = temp.find (old, 0);
            temp.erase (temp.find (old, 0), old.length());
            old = string("(") 
                + gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog)) 
                + string(")");
            temp.insert (loc, old);
            openldev_project_settings_set_lastfiles (openldev->project, temp.c_str());
            openldev_project_settings_write_project_file (openldev->project);
          }
        }
        
        openldev_file_manager_set_filename (openldev->notebook,
          gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog)));
        openldev_menu_file_save (openldev);
        break;
      }
      default:
        break;
    }
    
    gtk_widget_destroy (dialog);
  }
}

// Call the notebookmanager function to save all of the files that are
// currently open
void openldev_file_save_all (GtkMenuItem* menuitem, 
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  for (int i=0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (openldev->notebook->notebook)); i++)
  {
    // Save the file & set as unmodified
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW 
                                           (openldev->notebook->tabs[i]->textview));
    openldev_write_file (openldev->notebook->tabs[i]->fn,
                         openldev_file_get_text_in_buffer (GTK_SOURCE_BUFFER (buffer)));

    openldev_file_manager_mark_current_tab_modified (openldev->notebook, FALSE, i);
  }
}

// If the user chooses to allow it, revert to the saved version of a file. Then
// the syntax highlighting, action manager and line numbering are refreshed
void openldev_file_revert (GtkMenuItem* menuitem, 
                           gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
    if (openldev_question ("Are you sure you want to reload this file?\nAny unsaved changes will be lost."))
      openldev_file_manager_revert_to_saved (openldev->notebook);
}

void openldev_on_delete_tab (GtkWidget *widget,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  string temp, file;

  for (int i=0; i < openldev->notebook->tab_count; i++)
    if (openldev->notebook->tabs[i]->button == widget)
    {
      file = "(" + openldev->notebook->tabs[i]->fn + ")";
      if (openldev->project->lastfiles.find(file, 0) != string::npos)
      {
        temp = openldev->project->lastfiles;
        temp.erase (temp.find(file, 0), file.length());
        openldev_project_settings_set_lastfiles (openldev->project, temp.c_str());
        openldev_project_settings_write_project_file (openldev->project);
      }
    }
  
  openldev_file_manager_on_delete_tab (widget, (gpointer)openldev->notebook);
  openldev_manage_menu_items (openldev);
}

void openldev_manage_menu_items (OpenLDev *openldev)
{
  GtkWidget *item;
  gboolean show = openldev_file_manager_get_file_open (openldev->notebook);
  item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[0]);
  
  if (!(show == GTK_WIDGET_SENSITIVE(item)))
  {
    for (unsigned int i=0; i < NUM_MENUITEMS; i++)
    {
      item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[i]);
      gtk_widget_set_sensitive (item, show);
    }
  }
}
