/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-plugin-engine.h"

vector<string> openldev_get_installed_plugins (string);
string openldev_get_plugin_attribute (string, string);

OpenLDevPluginList* openldev_load_plugins_all (OpenLDev *openldev)
{
  g_assert (g_module_supported());
  
  OpenLDevPluginList *pl = new OpenLDevPluginList;
  string text, basedir = string (INSTALL_PREFIX) + "/lib/openldev/";
  vector<string> files = openldev_get_installed_plugins (basedir);
  
  for (unsigned int i=0; i < files.size(); i++)
  {
    OpenLDevPlugin *plugin = new OpenLDevPlugin;
    text = openldev_read_file (basedir + files[i]);
    
    plugin->name = openldev_get_plugin_attribute (text, "Name").c_str();
    plugin->author = openldev_get_plugin_attribute (text, "Author").c_str();
    plugin->copyright = openldev_get_plugin_attribute (text, "Copyright").c_str();
    plugin->description = openldev_get_plugin_attribute (text, "Description").c_str();
    plugin->library = g_strconcat (basedir.c_str(), "lib", openldev_get_plugin_attribute 
                                   (text, "Location").c_str(), ".la", NULL);
    
    plugin->file = files[i].c_str();
    plugin->enable = TRUE;
    plugin->plugin_id = i;
    plugin->state = PLUGIN_OK;
    
    plugin->module = g_module_open (plugin->library, G_MODULE_BIND_LAZY);
    if (!plugin->module)
      openldev_error ((gchar*) g_module_error ());
    else
    {
      if (!g_module_symbol (plugin->module, "init", (gpointer*)&plugin->init) ||
          !g_module_symbol (plugin->module, "destroy", (gpointer*)&plugin->destroy))
        openldev_error ((gchar*) g_module_error ());
      else
      {
        pl->plugins.push_back (plugin);
        pl->num_of_plugins++;
        plugin->init ((gpointer)openldev);
      }
    }
  }
  
  return pl;
}

string openldev_get_plugin_attribute (string text, string attr)
{
  int location = text.find (attr, 0) + attr.length() + 1;
  return text.substr (location, text.find (";", location) - location);
}

// Get a list of plugin files in the directory
vector<string> openldev_get_installed_plugins (string dir)
{
  vector<string> files;
  struct dirent **namelist;
  int n = scandir (dir.c_str(), &namelist, 0, alphasort), i;
  
  // Get the list of files by parsing through the list
  if (n >= 0)
  {
    while (n--) 
    {
      if (string (namelist[n]->d_name).find (".plugin", 0) != string::npos)
        files.push_back (namelist[n]->d_name);
      free (namelist[n]);
    } 
    free (namelist);
  }
  
  // Sort the vector in the opposite direction
  string temp;
  for (i=0; i < (signed)files.size() / 2; i++)
  {
    temp = files[i];
    files[i] = files[files.size() - i - 1];
    files[files.size() - i - 1] = temp;
  }
  
  return files;
}
