/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-settings.h"

#define PATH "/apps/openldev"

// List of keys that should be loaded
string keys [] = { "/auto_save", "/browser_width", "/output_height", "/width", "/height",
                   "/undo_actions", "/tab_size", "/auto_indent", "/auto_load", "/brackets",
                   "/line_numbers", "/replace_tabs", "/right_margin", "/wrap_enable",
                   "/wrap_split", "/error_color", "/warning_color", "/font_face",
                   "/font_size", "/last_project", "/recent_1", "/recent_2", "/recent_3",
                   "/recent_4", "recent_5" };

// The corresponding numbers to the keys
enum { AUTO_SAVE=0, BROWSER_WIDTH, OUTPUT_HEIGHT, WINDOW_WIDTH, WINDOW_HEIGHT,
       UNDO_ACTIONS, TAB_SIZE, AUTO_INDENT, AUTO_LOAD, BRACKETS, LINE_NUMBERS,
       REPLACE_TABS, RIGHT_MARGIN, WRAP_ENABLE, WRAP_SPLIT, ERROR_COLOR,
       WARNING_COLOR, FONT_FACE, FONT_SIZE, LAST_PROJECT, RECENT_1, RECENT_2,
       RECENT_3, RECENT_4, RECENT_5 };

void openldev_environment_load_settings_file (EnvironmentSettings*);
void openldev_environment_create_all (EnvironmentSettings*);
void openldev_environment_set_gboolean (EnvironmentSettings*, int, gboolean);
void openldev_environment_set_string (EnvironmentSettings*, int, const char*);
void openldev_environment_set_int (EnvironmentSettings*, int, int);

// Create a new environment settings structure
EnvironmentSettings* openldev_environment_settings_new ()
{
  EnvironmentSettings* settings = new EnvironmentSettings();
  
  settings->client = gconf_client_get_default ();
  openldev_environment_load_settings_file (settings);
  
  return settings;
}

// Load all of the environment settings from GConf
void openldev_environment_load_settings_file (EnvironmentSettings *settings)
{
  if (!gconf_client_dir_exists (settings->client, PATH, NULL))
    gconf_client_add_dir (settings->client, PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);
  
  openldev_environment_create_all (settings);
  
  settings->auto_save = gconf_client_get_int (settings->client, (PATH + keys[0]).c_str(), NULL);
  settings->browser_width = gconf_client_get_int (settings->client, (PATH + keys[1]).c_str(), NULL);
  settings->outputmanager_height = gconf_client_get_int (settings->client, (PATH + keys[2]).c_str(), NULL);
  settings->width = gconf_client_get_int (settings->client, (PATH + keys[3]).c_str(), NULL);
  settings->height = gconf_client_get_int (settings->client, (PATH + keys[4]).c_str(), NULL);
  settings->undo = gconf_client_get_int (settings->client, (PATH + keys[5]).c_str(), NULL);
  settings->tab_size = gconf_client_get_int (settings->client, (PATH + keys[6]).c_str(), NULL);
   
  settings->auto_indent = gconf_client_get_bool (settings->client, (PATH + keys[7]).c_str(), NULL);
  settings->auto_load = gconf_client_get_bool (settings->client, (PATH + keys[8]).c_str(), NULL);
  settings->brackets = gconf_client_get_bool (settings->client, (PATH + keys[9]).c_str(), NULL);
  settings->line_numbers = gconf_client_get_bool (settings->client, (PATH + keys[10]).c_str(), NULL);
  settings->replace_tabs = gconf_client_get_bool (settings->client, (PATH + keys[11]).c_str(), NULL);
  settings->right_margin = gconf_client_get_bool (settings->client, (PATH + keys[12]).c_str(), NULL);
  settings->wrap = gconf_client_get_bool (settings->client, (PATH + keys[13]).c_str(), NULL);
  settings->wrap_split = gconf_client_get_bool (settings->client, (PATH + keys[14]).c_str(), NULL);
   
  settings->error = gconf_client_get_string (settings->client, (PATH + keys[15]).c_str(), NULL);
  settings->warning = gconf_client_get_string (settings->client, (PATH + keys[16]).c_str(), NULL);
  settings->font_face = gconf_client_get_string (settings->client, (PATH + keys[17]).c_str(), NULL);
  settings->font_size = gconf_client_get_string (settings->client, (PATH + keys[18]).c_str(), NULL);
  settings->last_project = gconf_client_get_string (settings->client, (PATH + keys[19]).c_str(), NULL);
  
  // Get list of recent files
  settings->recent_files.push_back (gconf_client_get_string (settings->client, 
                                    (PATH + keys[20]).c_str(), NULL));
  settings->recent_files.push_back (gconf_client_get_string (settings->client, 
                                    (PATH + keys[21]).c_str(), NULL));
  settings->recent_files.push_back (gconf_client_get_string (settings->client, 
                                    (PATH + keys[22]).c_str(), NULL));
  settings->recent_files.push_back (gconf_client_get_string (settings->client, 
                                    (PATH + keys[23]).c_str(), NULL));
  settings->recent_files.push_back (gconf_client_get_string (settings->client, 
                                    (PATH + keys[24]).c_str(), NULL));
}

// Make sure that all of the settings already exist in GConf. If they don't, the
// default value is used.
void openldev_environment_create_all (EnvironmentSettings *settings)
{
  string key;
  int i;
  GConfValue *value;
  
  int default_int[] = { 10, 175, 175, 800, 600, 25, 2 };
  gboolean default_gboolean[] = { TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE };
  string default_str[] = { "#FF0000", "#0000CC", "Monospace", "10", "EMPTY", 
                           "", "", "", "", "" };
  
  // Check all integer elements
  for (i=0; i < 7; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_int (settings->client, key.c_str(), default_int[i], NULL);
    }
  }
  
  // Check all boolean elements
  for (i=7; i < 15; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir  (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_bool (settings->client, key.c_str(), default_gboolean[i-7], NULL);
    }
  }
  
  // Check all string elements
  for (i=15; i < 25; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_string (settings->client, key.c_str(), default_str[i-15].c_str(), NULL);
    }
  }
}

// These three functions set values of their type and then suggest
// as sync to GConf to make sure the changes are written.
void openldev_environment_set_gboolean (EnvironmentSettings *settings,
                                    int id, 
                                    gboolean value)
{
  gconf_client_set_bool (settings->client, (PATH + keys[id]).c_str(), 
                         value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

void openldev_environment_set_string (EnvironmentSettings *settings, 
                                      int id, 
                                      const char *value)
{
  gconf_client_set_string (settings->client, (PATH + keys[id]).c_str(), 
                           value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

void openldev_environment_set_int (EnvironmentSettings *settings, 
                                   int id, 
                                   int value)
{
  gconf_client_set_int (settings->client, (PATH + keys[id]).c_str(), 
                        value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

// Functions that allow the user to change environment settings
void openldev_environment_set_auto_indent (EnvironmentSettings* settings, 
                                           gboolean value)
{
  settings->auto_indent = value;
  openldev_environment_set_gboolean (settings, AUTO_INDENT, value);
}

void openldev_environment_set_auto_load (EnvironmentSettings* settings, 
                                         gboolean value)
{
  settings->auto_load = value;
  openldev_environment_set_gboolean (settings, AUTO_LOAD, value);
}

void openldev_environment_set_brackets (EnvironmentSettings* settings, 
                                        gboolean value)
{
  settings->brackets = value;
  openldev_environment_set_gboolean (settings, BRACKETS, value);
}

void openldev_environment_set_line_numbers (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->line_numbers = value;
  openldev_environment_set_gboolean (settings, LINE_NUMBERS, value);
}

void openldev_environment_set_replace_tabs (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->replace_tabs = value;
  openldev_environment_set_gboolean (settings, REPLACE_TABS, value);
}

void openldev_environment_set_right_margin (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->right_margin = value;
  openldev_environment_set_gboolean (settings, RIGHT_MARGIN, value);
}

void openldev_environment_set_wrap (EnvironmentSettings* settings, 
                                    gboolean value)
{
  settings->wrap = value;
  openldev_environment_set_gboolean (settings, WRAP_ENABLE, value);
}

void openldev_environment_set_wrap_split (EnvironmentSettings* settings, 
                                          gboolean value)
{
  settings->wrap_split = value;
  openldev_environment_set_gboolean (settings, WRAP_SPLIT, value);
}

void openldev_environment_set_auto_save (EnvironmentSettings* settings, 
                                         int value)
{
  settings->auto_save = value;
  openldev_environment_set_int (settings, AUTO_SAVE, value);
}

void openldev_environment_set_browser_width (EnvironmentSettings* settings, 
                                             int value)
{
  settings->browser_width = value;
  openldev_environment_set_int (settings, BROWSER_WIDTH, value);
}

void openldev_environment_set_height (EnvironmentSettings* settings, 
                                      int value)
{
  settings->height = value;
  openldev_environment_set_int (settings, WINDOW_HEIGHT, value);
}

void openldev_environment_set_outputmanager_height (EnvironmentSettings* settings, 
                                                    int value)
{
  settings->outputmanager_height = value;
  openldev_environment_set_int (settings, OUTPUT_HEIGHT, value);
}

void openldev_environment_set_tab_size (EnvironmentSettings* settings, 
                                        int value)
{
  settings->tab_size = value;
  openldev_environment_set_int (settings, TAB_SIZE, value);
}

void openldev_environment_set_undo (EnvironmentSettings* settings, 
                                    int value)
{
  settings->undo = value;
  openldev_environment_set_int (settings, UNDO_ACTIONS, value);
}

void openldev_environment_set_width (EnvironmentSettings* settings, 
                                     int value)
{
  settings->width = value;
  openldev_environment_set_int (settings, WINDOW_WIDTH, value);
}

void openldev_environment_set_error (EnvironmentSettings* settings, 
                                     const char* value)
{
  settings->error = value;
  openldev_environment_set_string (settings, ERROR_COLOR, value);
}

void openldev_environment_set_font_face (EnvironmentSettings* settings, 
                                         const char* value)
{
  settings->font_face = value;
  openldev_environment_set_string (settings, FONT_FACE, value);
}

void openldev_environment_set_font_size (EnvironmentSettings* settings, 
                                         const char* value)
{
  settings->font_size = value;
  openldev_environment_set_string (settings, FONT_SIZE, value);
}

void openldev_environment_set_last_project (EnvironmentSettings* settings, 
                                            const char* value)
{
  settings->last_project = value;
  openldev_environment_set_string (settings, LAST_PROJECT, value);
}

void openldev_environment_set_warning (EnvironmentSettings* settings, 
                                       const char* value)
{
  settings->warning = value;
  openldev_environment_set_string (settings, WARNING_COLOR, value);
}

void openldev_environment_add_recent_file (EnvironmentSettings *settings, 
                                           const char *value)
{
  if (settings->recent_files [0] != value &&
      settings->recent_files [1] != value &&
      settings->recent_files [2] != value &&
      settings->recent_files [3] != value &&
      settings->recent_files [4] != value)
  {
    settings->recent_files[4] = settings->recent_files[3];
    settings->recent_files[3] = settings->recent_files[2];
    settings->recent_files[2] = settings->recent_files[1];
    settings->recent_files[1] = settings->recent_files[0];
    settings->recent_files[0] = value;
  
    openldev_environment_set_string (settings, RECENT_1, settings->recent_files[0].c_str());
    openldev_environment_set_string (settings, RECENT_2, settings->recent_files[1].c_str());
    openldev_environment_set_string (settings, RECENT_3, settings->recent_files[2].c_str());
    openldev_environment_set_string (settings, RECENT_4, settings->recent_files[3].c_str());
    openldev_environment_set_string (settings, RECENT_5, settings->recent_files[4].c_str());
  }
}

const char* openldev_environment_get_recent_short (EnvironmentSettings *settings,
                                                   int item)
{
  string temp = settings->recent_files[item];
  while (temp.find ("/", 0) != string::npos)
    temp.erase (0, temp.find ("/", 0) + 1);
  return temp.c_str();
}
