/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-output-manager.h"

// Create the output manager and add the default elements
OutputManager* openldev_output_manager_new ()
{
  OutputManager *manager = new OutputManager ();
  
  manager->search = openldev_output_search_new ();
  manager->build = openldev_output_build_new ();
  manager->terminal = openldev_output_terminal_new ();
  manager->notebook = gtk_notebook_new ();
  
  // Create the headers for the tabs
  manager->compiler = gtk_hbox_new (FALSE, 0);
  manager->compiler_label = gtk_label_new ("Compiler");
  manager->compiler_image = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_MENU);
  manager->findtab = gtk_hbox_new (FALSE, 0);
  manager->findtab_label = gtk_label_new ("Search");
  manager->findtab_image = gtk_image_new_from_stock ("gtk-find", GTK_ICON_SIZE_MENU);
  manager->terminaltab = gtk_hbox_new (FALSE, 0);
  manager->terminaltab_label = gtk_label_new ("Terminal");
  manager->terminaltab_image = gtk_image_new_from_stock ("gtk-justify-left", GTK_ICON_SIZE_MENU);
  
  gtk_box_set_spacing (GTK_BOX(manager->compiler), 5);
  gtk_box_pack_start (GTK_BOX(manager->compiler), manager->compiler_image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(manager->compiler), manager->compiler_label, FALSE, FALSE, 0);
  gtk_box_set_spacing (GTK_BOX(manager->findtab), 5);
  gtk_box_pack_start (GTK_BOX(manager->findtab), manager->findtab_image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(manager->findtab), manager->findtab_label, FALSE, FALSE, 0);
  gtk_box_set_spacing (GTK_BOX(manager->terminaltab), 5);
  gtk_box_pack_start (GTK_BOX(manager->terminaltab), manager->terminaltab_image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(manager->terminaltab), manager->terminaltab_label, FALSE, FALSE, 0);
  
  gtk_notebook_append_page (GTK_NOTEBOOK (manager->notebook),
                            openldev_output_build_get_scrolled_window (manager->build),
                            manager->compiler);
  gtk_notebook_append_page (GTK_NOTEBOOK (manager->notebook),
                            openldev_output_search_get_scrolled_window (manager->search),
                            manager->findtab);
  gtk_notebook_append_page (GTK_NOTEBOOK (manager->notebook),
                            openldev_output_terminal_get_hbox (manager->terminal),
                            manager->terminaltab);
  
  gtk_widget_show (manager->compiler);
  gtk_widget_show (manager->compiler_image);
  gtk_widget_show (manager->compiler_label);
  gtk_widget_show (manager->findtab);
  gtk_widget_show (manager->findtab_image);
  gtk_widget_show (manager->findtab_label);
  gtk_widget_show (manager->terminaltab);
  gtk_widget_show (manager->terminaltab_image);
  gtk_widget_show (manager->terminaltab_label);
  gtk_widget_show (manager->notebook);
  
  return manager;
}

OutputBuild* openldev_output_manager_get_build (OutputManager *manager)
{ return manager->build;
}

OutputSearch* openldev_output_manager_get_search (OutputManager *manager)
{ return manager->search;
}

OutputTerminal* openldev_output_manager_get_terminal (OutputManager *manager)
{ return manager->terminal;
}
