/***************************************************************************
 *   Copyright (C) 2006 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "makefile-editor-edit.h"

int num_prefixes = 0;
string makefilename = "";

void makefile_editor_edit_purge_element (Makefile*, int);

// Target dialog population functions
void makefile_editor_edit_target_data (Makefile*, int, gchar*, GladeXML*);
int makefile_editor_edit_target_get_type (Makefile*, string, int&);
void makefile_editor_edit_target_populate_sources (GtkWidget*, Makefile*, string);
vector<string> makefile_editor_edit_target_get_sources (Makefile*, string);
gboolean makefile_editor_edit_get_source_in_target (Makefile*, string, string);
void makefile_editor_edit_target_set_data (GtkWidget*, Makefile*, int);
void makefile_editor_edit_target_set_prefix (string, GtkWidget*);
void makefile_editor_edit_target_program_or_library (Makefile*, GladeXML*, gchar*,
                                                     gchar*, int, int);

// Target dialog saving function
void makefile_editor_edit_target_purge_target (Makefile*, string);
void makefile_editor_edit_target_purge_prefix (Makefile*, string);
void makefile_editor_edit_target_add_element (Makefile*, string, string);

// Target dialog signals
void makefile_editor_edit_target_toggle (GtkToggleButton*, gpointer);
void makefile_editor_edit_target_type_changed (GtkComboBox*, gpointer);
void makefile_editor_edit_target_prefix_changed (GtkComboBox*, gpointer);
void makefile_editor_edit_source_toggled (GtkCellRendererToggle*, gchar*, GtkTreeModel*);
void makefile_editor_edit_target_cancel (GtkButton*, gpointer);
void makefile_editor_edit_target_apply (GtkButton*, gpointer);

// This handles finding the type of object selected & calling the correct edit function
void makefile_editor_edit (MakefileEditor *editor)
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->content));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    int type;
    gtk_tree_model_get (model, &iter, 2, &type, -1);
    
    // Call the edit function
    if (type == MAKEFILE_EDITOR_VARIABLE)
    {
      if (makefile_editor_edit_variable (editor, iter, model))
        makefile_editor_populate (editor);
    }
    else if (type == MAKEFILE_EDITOR_TARGET)
    {
      if (makefile_editor_edit_target (editor, iter, model))
        makefile_editor_populate (editor);
    }
    else if (type == MAKEFILE_EDITOR_BASE || type == MAKEFILE_EDITOR_SUBPROJECT)
    {
      if (makefile_editor_edit_subproject (editor, iter, model))
        makefile_editor_populate (editor);
    }
    else if (type == MAKEFILE_EDITOR_SCRIPT)
    {
      if (makefile_editor_edit_script (editor, iter, model))
        makefile_editor_populate (editor);
    }
  }
}

void makefile_editor_edit_purge_element (Makefile *mf, 
                                         int i)
{
  vector<int>::iterator iiter = mf->types.begin();
  vector<MakefileAttr*>::iterator miter = mf->attributes.begin();
  vector<string>::iterator siter = mf->names.begin();
  
  for (int j=0; j < i; j++)
  {
    iiter++;
    miter++;
    siter++;
  }
  
  mf->types.erase (iiter);
  mf->attributes.erase (miter);
  mf->names.erase (siter);
}

/************************ Edit Target ************************/

// Edit a target object
gboolean makefile_editor_edit_target (MakefileEditor *editor,
                                  GtkTreeIter iter,
                                  GtkTreeModel *model)
{
  gboolean type_is_data = FALSE;
  unsigned int i;
  num_prefixes = 0;
  string search, base;
  gchar *makefile = g_strconcat (editor->project->location.c_str(), 
                                 editor->current_path.c_str(),
                                 "Makefile.am", NULL);
  makefilename = makefile;
  gchar *gladefile = g_strconcat (INSTALL_PREFIX, "/share/openldev/glade/",
                                 "mfeditor-target.glade", NULL);
  Makefile *mf = openldev_makefile_parser_new (makefile);
  
  // Get the name of the target in question
  gchar *target;
  gtk_tree_model_get (model, &iter, 1, &target, -1);
  base = target;
  
  // Load this dialog because it might be necessary
  GladeXML *xml = glade_xml_new (gladefile, NULL, NULL);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  GtkWidget *type = glade_xml_get_widget (xml, "type");
  GtkWidget *prefix = glade_xml_get_widget (xml, "prefix");
  GtkWidget *sources = glade_xml_get_widget (xml, "sources");
  GtkWidget *uselocation = glade_xml_get_widget (xml, "alternative_location");
  GtkWidget *cancel = glade_xml_get_widget (xml, "cancel");
  GtkWidget *apply = glade_xml_get_widget (xml, "apply");
  
  g_signal_connect (G_OBJECT (uselocation), "toggled",
                    G_CALLBACK (makefile_editor_edit_target_toggle), 
                    (gpointer) xml);
  g_signal_connect (G_OBJECT (type), "changed",
                    G_CALLBACK (makefile_editor_edit_target_type_changed), 
                    (gpointer) xml);
  g_signal_connect (G_OBJECT (prefix), "changed",
                    G_CALLBACK (makefile_editor_edit_target_prefix_changed), 
                    (gpointer) xml);
  g_signal_connect (G_OBJECT (cancel), "clicked",
                    G_CALLBACK (makefile_editor_edit_target_cancel),
                    (gpointer) xml);
  g_signal_connect (G_OBJECT (apply), "clicked",
                    G_CALLBACK (makefile_editor_edit_target_apply),
                    (gpointer) xml);
  
  // Check for targetdir (this would be a "Data" type)
  search = g_strconcat (target, "dir", NULL);
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == search)
    {
      makefile_editor_edit_target_data (mf, i, target, xml);
      type_is_data = TRUE;
      break;
    }
  
  if (!type_is_data)
  {
    int attr_location;
    int attrtype = makefile_editor_edit_target_get_type (mf, target, attr_location);
  
    // We need to remove the dash and period characters for libraries
    for (i=0; i < base.length(); i++)
      if (base[i] == '-' || base[i] == '.')
        base[i] = '_';
    
    makefile_editor_edit_target_program_or_library (mf, xml, target, (gchar*)
                                base.c_str(), attrtype, attr_location);
  }
  
  makefile_editor_edit_target_populate_sources (sources, mf, base);
  
  g_free (target);
  g_free (makefile);
  g_free (gladefile);
  
  int result = 0;
  while (result == 0)
    result = gtk_dialog_run (GTK_DIALOG (dialog));
  
  return TRUE;
}

// Add the information necessary to be a program or library target
void makefile_editor_edit_target_program_or_library (Makefile *mf,
                                                     GladeXML *xml,
                                                     gchar *target,
                                                     gchar *base,
                                                     int attrtype,
                                                     int attr_location)
{
  unsigned int i, j;
  
  GtkWidget *name = glade_xml_get_widget (xml, "name");
  GtkWidget *ldflags = glade_xml_get_widget (xml, "ldflags");
  GtkWidget *ldadd = glade_xml_get_widget (xml, "ldadd");
  GtkWidget *type = glade_xml_get_widget (xml, "type");
  GtkWidget *prefix = glade_xml_get_widget (xml, "prefix");
  GtkWidget *dataname = glade_xml_get_widget (xml, "dataname");
  GtkWidget *uselocation = glade_xml_get_widget (xml, "alternative_location");
  GtkWidget *location = glade_xml_get_widget (xml, "location");
  GtkWidget *data = glade_xml_get_widget (xml, "data");
  GtkWidget *original_target = glade_xml_get_widget (xml, "original_target");
  GtkWidget *original_prefix = glade_xml_get_widget (xml, "original_prefix");
  
  gtk_entry_set_text (GTK_ENTRY (name), target);
  gtk_entry_set_text (GTK_ENTRY (original_target), target);
    
  // Search for & set the LDFLAGS
  string search = g_strconcat (base, "_LDFLAGS", NULL);
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == search)
    {
      if (mf->attributes[i]->attr.size() > 0)
        gtk_entry_set_text (GTK_ENTRY (ldflags), mf->attributes[i]->attr[0].c_str());
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        gtk_entry_set_text (GTK_ENTRY (ldflags), g_strconcat (gtk_entry_get_text
                           (GTK_ENTRY (ldflags)), " ", 
                           mf->attributes[i]->attr[j].c_str(), NULL));
      break;
    }
    
  // Set the LDADD or LIBADD depending on type
  if (attrtype == MAKEFILE_TYPE_PROGRAM)
    search = g_strconcat (base, "_LDADD", NULL);
  else
    search = g_strconcat (base, "_LIBADD", NULL);
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == search)
    {
      if (mf->attributes[i]->attr.size() > 0)
        gtk_entry_set_text (GTK_ENTRY (ldadd), mf->attributes[i]->attr[0].c_str());
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        gtk_entry_set_text (GTK_ENTRY (ldadd), g_strconcat (gtk_entry_get_text
                           (GTK_ENTRY (ldadd)), " ", 
                           mf->attributes[i]->attr[j].c_str(), NULL));
      break;
    }
  
  // Set the type
  if (attrtype == MAKEFILE_TYPE_PROGRAM)
    gtk_combo_box_set_active (GTK_COMBO_BOX (type), 0);
  else if (attrtype == MAKEFILE_TYPE_LIBRARY)
    gtk_combo_box_set_active (GTK_COMBO_BOX (type), 1);
  else if (attrtype == MAKEFILE_TYPE_LTLIBRARY)
    gtk_combo_box_set_active (GTK_COMBO_BOX (type), 2);
    
  // Check the prefix and load data if necessary
  if (attr_location >= 0)
  {
    string pfx = mf->names[attr_location];
    pfx = pfx.substr (0, pfx.find ("_", 0));
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 4 + ((4 - num_prefixes) * -1));
    
    for (i=0; i < (unsigned) num_prefixes; i++)
      makefile_editor_edit_target_set_prefix (pfx, prefix);
    
    if (string (gtk_combo_box_get_active_text (GTK_COMBO_BOX (prefix))) == "Use Data Prefix")
    {
      string dir = pfx + "dir", dta = pfx + "_DATA";
      gtk_entry_set_text (GTK_ENTRY (dataname), pfx.c_str());
      gtk_entry_set_text (GTK_ENTRY (original_prefix), pfx.c_str());
      
      for (i=0; i < mf->types.size(); i++)
      {
        if (mf->names[i] == dir)
        {
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (uselocation), TRUE);
          if (mf->attributes[i]->attr.size() > 0)
            gtk_entry_set_text (GTK_ENTRY (location), mf->attributes[i]->attr[0].c_str());
        }
        else if (mf->names[i] == dta)
          makefile_editor_edit_target_set_data (data, mf, i);
      }
    }
  }
}

// Reset the choice of prefix for a program or library if it is
// not using a data prefix
void makefile_editor_edit_target_set_prefix (string pfx,
                                             GtkWidget *prefix)
{
  if (pfx == "bin")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 0);
  else if (pfx == "sbin")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 1);
  else if (pfx == "libexec")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 2);
  else if (pfx == "lib")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 0);
  else if (pfx == "noinst")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 1 + ((4 - num_prefixes) * -1));
  else if (pfx == "pkglib")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 2 + ((4 - num_prefixes) * -1));
  else if (pfx == "check")
    gtk_combo_box_set_active (GTK_COMBO_BOX (prefix), 3 + ((4 - num_prefixes) * -1));
}

// Set the text that is found in a data type
void makefile_editor_edit_target_set_data (GtkWidget *data, 
                                           Makefile *mf, 
                                           int i)
{
  if (mf->attributes[i]->attr.size() > 0)
    gtk_entry_set_text (GTK_ENTRY (data), mf->attributes[i]->attr[0].c_str());
  for (unsigned int j=1; j < mf->attributes[i]->attr.size(); j++)
    gtk_entry_set_text (GTK_ENTRY (data), g_strconcat (gtk_entry_get_text
                        (GTK_ENTRY (data)), " ", 
                        mf->attributes[i]->attr[j].c_str(), NULL));
}

// Add the list of sources and whether they are a part of the current target
void makefile_editor_edit_target_populate_sources (GtkWidget *sources, 
                                                   Makefile *mf, 
                                                   string base)
{
  vector<string> files = makefile_editor_edit_target_get_sources (mf, base);
  GtkTreeIter row;
  GtkCellRenderer *renderer;
  GtkTreeModel *model = GTK_TREE_MODEL (gtk_tree_store_new (4, G_TYPE_BOOLEAN,
                                        G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING));
                                                          
  gtk_tree_view_set_model (GTK_TREE_VIEW (sources), model);
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (sources));

  // This gave me a lot of trouble because it is late right now, and therefore, I am stupid
  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set (renderer, "xalign", 0.5, NULL);
  g_signal_connect (renderer, "toggled", 
                    G_CALLBACK (makefile_editor_edit_source_toggled), model);
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (sources), 0, "",
							      renderer, "active", 0, "visible", 1, "activatable", 2, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (sources), 1, "Source Files",
							      renderer, "text", 3, NULL);
  
  // Make sure to check whether or not to set the target active
  for (unsigned int i=0; i < files.size(); i++)
  {
    gtk_tree_store_append (GTK_TREE_STORE (model), &row, NULL);
    gtk_tree_store_set (GTK_TREE_STORE (model), &row, 0,
                        makefile_editor_edit_get_source_in_target (mf, base, files[i]),
                        1, TRUE, 2, TRUE, 3, files[i].c_str(), -1);
  }
  
  g_object_unref (model);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (sources), FALSE);
}

// Check if the current source file is a part of the sources list for the target
gboolean makefile_editor_edit_get_source_in_target (Makefile *mf, 
                                                    string base, 
                                                    string file)
{
  string sources = base + "_SOURCES";
  unsigned int i, j;
  
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == sources)
    {
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        if (mf->attributes[i]->attr[j] == file)
          return TRUE;
      break;
    }
  
  return FALSE;
}

// Toggle the source check button because the user said to do it
void makefile_editor_edit_source_toggled (GtkCellRendererToggle *renderer, 
                                          gchar *path_string, 
                                          GtkTreeModel *model)
{
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;
  gboolean toggled;
  gint *column = (gint*) g_object_get_data (G_OBJECT (renderer), "column");

  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, column, &toggled, -1);
  gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, !toggled, -1);
  gtk_tree_path_free (path);
}

// Get a list of C/C++ source files in the directory
vector<string> makefile_editor_edit_target_get_sources (Makefile *mf,
                                                        string base)
{
  string str, substr, temp;
  base += "_SOURCES";
  vector<string> files;
  struct dirent **namelist;
  
  str = mf->filename.substr (0, mf->filename.find ("Makefile.am", 0));
  int n = scandir (str.c_str(), &namelist, 0, alphasort);
  
  // Get the list of files by parsing through the list
  if (n >= 0)
  {
    while (n--) 
    {
      str = namelist[n]->d_name;
      
      if (str.find (".", 0) != string::npos)
      {
        substr = str.substr (str.find (".", 0), str.length() - str.find (".", 0));
      
        if (substr == ".c" || substr == ".cc" || substr == ".cxx" ||
            substr == ".cpp" || substr == ".hh" || substr == ".h" ||
            substr == ".hpp" || substr == ".hxx")
          files.push_back (namelist[n]->d_name);
        free (namelist[n]);
      }
    } 
    free (namelist);
  }
  
  for (unsigned int i=0; i < files.size() / 2; i++)
  {
    temp = files[i];
    files[i] = files[files.size()-i-1];
    files[files.size()-i-1] = temp;
  }
  
  return files;
}

// Toggle the sensitivity of the location entry
void makefile_editor_edit_target_toggle (GtkToggleButton *togglebutton,
                                         gpointer data)
{
  if (GTK_WIDGET_SENSITIVE (GTK_WIDGET (togglebutton)))
  {
    GladeXML *xml = (GladeXML*) data;
    GtkWidget *location = glade_xml_get_widget (xml, "location");
  
    if (gtk_toggle_button_get_active (togglebutton))
      gtk_widget_set_sensitive (location, TRUE);
    else
      gtk_widget_set_sensitive (location, FALSE);
  }
}

// The type has changed! That means we must perform some actions.
void makefile_editor_edit_target_type_changed (GtkComboBox *type,
                                               gpointer user_data)
{
  GladeXML *xml = (GladeXML*) user_data;
  GtkComboBox *prefix = GTK_COMBO_BOX (glade_xml_get_widget (xml, "prefix"));
  GtkWidget *name = glade_xml_get_widget (xml, "name");
  GtkWidget *ldflags = glade_xml_get_widget (xml, "ldflags");
  GtkWidget *ldadd = glade_xml_get_widget (xml, "ldadd");
  GtkWidget *sources = glade_xml_get_widget (xml, "sources");
  GtkWidget *data = glade_xml_get_widget (xml, "data");
  GtkWidget *dataname = glade_xml_get_widget (xml, "dataname");
  GtkWidget *uselocation = glade_xml_get_widget (xml, "alternative_location");
  string text = gtk_combo_box_get_active_text (type);
  
  // Remove all of the prefixes in the combo box
  while (num_prefixes > 0)
  {
    gtk_combo_box_remove_text (prefix, 0);
    num_prefixes--;
  }
  
  // There is a data type selected, so make sure only the
  // data widgets are sensitive
  if (text == "Data")
  {
    num_prefixes = 0;
    gtk_widget_set_sensitive (name, FALSE);
    gtk_widget_set_sensitive (ldflags, FALSE);
    gtk_widget_set_sensitive (ldadd, FALSE);
    gtk_widget_set_sensitive (sources, FALSE);
    gtk_widget_set_sensitive (uselocation, TRUE);
    gtk_widget_set_sensitive (data, TRUE);
    gtk_widget_set_sensitive (dataname, TRUE);
    gtk_combo_box_set_active (prefix, 0);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (uselocation), TRUE);
  }
  // Otherwise we must enable everything
  else if (!text.empty())
  {
    gtk_combo_box_prepend_text (prefix, "check");
    gtk_combo_box_prepend_text (prefix, "pkglib");
    gtk_combo_box_prepend_text (prefix, "noinst");  
  
    if (text == "Program")
    {
      num_prefixes = 6;
      gtk_combo_box_prepend_text (prefix, "libexec");
      gtk_combo_box_prepend_text (prefix, "sbin");
      gtk_combo_box_prepend_text (prefix, "bin");
      gtk_widget_set_sensitive (name, TRUE);
      gtk_widget_set_sensitive (ldflags, TRUE);
      gtk_widget_set_sensitive (ldadd, TRUE);
      gtk_widget_set_sensitive (sources, TRUE);
    }
    else if (text == "Library" || text == "Libtool Library")
    {
      num_prefixes = 4;
      gtk_combo_box_prepend_text (prefix, "lib");
    }
    
    gtk_widget_set_sensitive (name, TRUE);
    gtk_widget_set_sensitive (ldflags, TRUE);
    gtk_widget_set_sensitive (ldadd, TRUE);
    gtk_widget_set_sensitive (sources, TRUE);
  }
  // Another otherwise, everything is wrong!
  else
    openldev_error ("You must choose a target type.");
}

// The prefix has changes, so we must set the sensitivity based up it
void makefile_editor_edit_target_prefix_changed (GtkComboBox *prefix,
                                                 gpointer data)
{
  GladeXML *xml = (GladeXML*) data;
  GtkWidget *type = glade_xml_get_widget (xml, "type");
  GtkWidget *dataname = glade_xml_get_widget (xml, "dataname");
  GtkWidget *datawidget = glade_xml_get_widget (xml, "data");
  GtkWidget *uselocation = glade_xml_get_widget (xml, "alternative_location");
  GtkWidget *location = glade_xml_get_widget (xml, "location");
  string type_text = gtk_combo_box_get_active_text (GTK_COMBO_BOX (type)),
         prefix_text = gtk_combo_box_get_active_text (prefix);
  
  gboolean test1 = (type_text == "Data");
  gboolean test2 = (prefix_text == "Use Data Prefix");
  
  // If Data type is not selected, we must enable or disable the data
  // widgets if the user wants or doesn't want to use them respectively.
  if (!test1)
  {
    gtk_widget_set_sensitive (dataname, test2);
    gtk_widget_set_sensitive (datawidget, test2);
    gtk_widget_set_sensitive (uselocation, test2);
    gtk_widget_set_sensitive (location, test2);
  }
}

// Retrieve what type the target happens to be
int makefile_editor_edit_target_get_type (Makefile *mf,
                                          string target,
                                          int &location)
{
  unsigned int i, j;
  
  for (i=0; i < mf->types.size(); i++)
  {
    if (mf->names[i].find ("_PROGRAMS", 0) != string::npos)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        if (mf->attributes[i]->attr[j] == target)
        {
          location = i;
          return MAKEFILE_TYPE_PROGRAM;
        }
    if (mf->names[i].find ("_LIBRARIES", 0) != string::npos)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        if (mf->attributes[i]->attr[j] == target)
        {
          location = i;
          return MAKEFILE_TYPE_LIBRARY;
        }
    if (mf->names[i].find ("_LTLIBRARIES", 0) != string::npos)
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
        if (mf->attributes[i]->attr[j] == target)
        {
          location = i;
          return MAKEFILE_TYPE_LTLIBRARY;
        }
  }
  
  location = -1;
  return MAKEFILE_TYPE_ERROR;
}

// This is run when a target is of type "Data"
void makefile_editor_edit_target_data (Makefile *mf,
                                       int i,
                                       gchar *target, 
                                       GladeXML *xml)
{
  GtkWidget *dataname = glade_xml_get_widget (xml, "dataname");
  GtkWidget *uselocation = glade_xml_get_widget (xml, "alternative_location");
  GtkWidget *location = glade_xml_get_widget (xml, "location");
  GtkWidget *type = glade_xml_get_widget (xml, "type");
  GtkWidget *data = glade_xml_get_widget (xml, "data");
  GtkWidget *original_prefix = glade_xml_get_widget (xml, "original_prefix");

  gtk_entry_set_text (GTK_ENTRY (original_prefix), target);
  gtk_entry_set_text (GTK_ENTRY (dataname), target);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (uselocation), TRUE);
      
  if (mf->attributes[i]->attr.size() > 0)
    gtk_entry_set_text (GTK_ENTRY (location), mf->attributes[i]->attr[0].c_str());
  gtk_combo_box_set_active (GTK_COMBO_BOX (type), 3);
  
  // Populate the list of data files
  string search = g_strconcat (target, "_DATA", NULL);
  for (unsigned int j=0; j < mf->types.size(); j++)
    if (mf->names[j] == search)
    {
      makefile_editor_edit_target_set_data (data, mf, j);
      break;
    }
}

// The user doesn't want to apply their changes, so destroy the dialog
void makefile_editor_edit_target_cancel (GtkButton *button, 
                                         gpointer data)
{ 
  GtkWidget *dialog = glade_xml_get_widget ((GladeXML*) data, "dialog");
  gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
  gtk_widget_destroy (dialog);
}

// Apply the changes the user made unless there are conflicts
void makefile_editor_edit_target_apply (GtkButton *button, 
                                        gpointer user_data)
{
  string error;
  GladeXML *xml = (GladeXML*) user_data;
  GtkComboBox *type = GTK_COMBO_BOX (glade_xml_get_widget (xml, "type"));
  GtkComboBox *prefix = GTK_COMBO_BOX (glade_xml_get_widget (xml, "prefix"));
  GtkToggleButton *uselocation = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "alternative_location"));
  GtkTreeView *sources = GTK_TREE_VIEW (glade_xml_get_widget (xml, "sources"));
  GtkEntry *location = GTK_ENTRY (glade_xml_get_widget (xml, "location"));
  GtkEntry *dataname = GTK_ENTRY (glade_xml_get_widget (xml, "dataname"));
  GtkEntry *data = GTK_ENTRY (glade_xml_get_widget (xml, "data"));
  GtkEntry *name = GTK_ENTRY (glade_xml_get_widget (xml, "name"));
  GtkEntry *ldadd = GTK_ENTRY (glade_xml_get_widget (xml, "ldadd"));
  GtkEntry *ldflags = GTK_ENTRY (glade_xml_get_widget (xml, "ldflags"));
  GtkEntry *original_target = GTK_ENTRY (glade_xml_get_widget (xml, "original_target"));
  GtkEntry *original_prefix = GTK_ENTRY (glade_xml_get_widget (xml, "original_prefix"));
  
  // Retrieve the text information entered in the widgets
  Makefile *mf = openldev_makefile_parser_new (makefilename.c_str());
  string prefix_text = gtk_combo_box_get_active_text (prefix),
         type_text = gtk_combo_box_get_active_text (type),
         location_text = gtk_entry_get_text (location),
         dataname_text = gtk_entry_get_text (dataname),
         data_text = gtk_entry_get_text (data),
         name_text = gtk_entry_get_text (name),
         purge_target = gtk_entry_get_text (original_target),
         purge_prefix = gtk_entry_get_text (original_prefix),
         ldadd_text = gtk_entry_get_text (ldadd),
         ldflags_text = gtk_entry_get_text (ldflags);
  gboolean uselocation_choice = gtk_toggle_button_get_active (uselocation);
  
  // Check for errors & return if there are any
  if (type_text.empty())
    error = "You must choose a target type!";
  else if (prefix_text.empty())
    error = "You must choose a target prefix!";
  else if (prefix_text == "Use Data Prefix")
  {
    if (dataname_text.empty())
      error = "You must specify a data prefix!";
    else if (!uselocation_choice || location_text.empty())
      error = "You must specify a data location!";
    else if (type_text == "Data" && data_text.empty())
      error = "You must specify data to be installed!";
  }
  else if (type_text != "Data")
    if (name_text.empty())
      error = "You must specify a target name!";
  if (!error.empty())
  {
    openldev_error ((gchar*) error.c_str());
    return;
  }
  
  if (!purge_target.empty())
    makefile_editor_edit_target_purge_target (mf, purge_target);
  if (!purge_prefix.empty())
    makefile_editor_edit_target_purge_prefix (mf, purge_prefix);
  
  // Handle information that is specific for programs & libraries
  if (type_text != "Data")
  {    
    string search;
    unsigned int i;
    gboolean name_set_done = FALSE;
    
    // Set the correct prefix
    if (prefix_text == "Use Data Prefix")
      search = dataname_text;
    else
      search = prefix_text;
    
    // Set the correct ending
    if (type_text == "Libtool Library")
      search += "_LTLIBRARIES";
    else if (type_text == "Library")
      search += "_LIBRARIES";
    else
      search += "_PROGRAMS";
    
    // If it already exists, just add the element
    for (i=0; i < mf->types.size(); i++)
      if (mf->names[i] == search)
      {
        mf->attributes[i]->attr.push_back (name_text);
        name_set_done = TRUE;
        break;
      }
    
    // Otherwise, just add it as a new element
    if (!name_set_done)
      makefile_editor_edit_target_add_element (mf, search, name_text); 
    
    // Make sure there are no dots or dashes
    string base = name_text;
    for (i=0; i < base.length(); i++)
      if (base[i] == '-' || base[i] == '.')
        base[i] = '_';
    
    // Add LDADD or LIBADD elements
    if (!ldadd_text.empty())
    {
      if (type_text == "Program")
        makefile_editor_edit_target_add_element (mf, (base + "_LDADD"), ldadd_text);
      else
        makefile_editor_edit_target_add_element (mf, (base + "_LIBADD"), ldadd_text);
    }
    
    // Add the LDFLAGS element
    if (!ldflags_text.empty())
      makefile_editor_edit_target_add_element (mf, (base + "_LDFLAGS"), ldflags_text);
    
    int srclocation = mf->types.size();
    makefile_editor_edit_target_add_element (mf, (base + "_SOURCES"), "");
    
    // Add the sources to the Makefile.am
    GtkTreeModel *model = gtk_tree_view_get_model (sources);
    GtkTreeIter iter;
    for (i=0; i < (unsigned) gtk_tree_model_iter_n_children (model, NULL); i++)
    {
      gboolean show;
      gchar *file;
      gtk_tree_model_iter_nth_child (model, &iter, NULL, i);
      gtk_tree_model_get (model, &iter, 0, &show, 3, &file, -1);
      
      if (show)
        mf->attributes[srclocation]->attr.push_back (file);
    }
  }
  
  // Write all of the information for something using a data prefix
  if (prefix_text == "Use Data Prefix")
  {
    // Clean up from before & add the location
    makefile_editor_edit_target_add_element (mf, (dataname_text + "dir"), location_text);
    
    // Add data to install as well if the user chooses
    if (!data_text.empty())
    {
      gboolean data_added = FALSE;
      makefile_editor_edit_target_add_element (mf, (dataname_text + "_DATA"), data_text);
      for (unsigned int j=0; j < mf->types.size(); j++)
        if (mf->names[j] == "EXTRA_DIST")
        {
          mf->attributes[j]->attr.push_back ("$(" + dataname_text + "_DATA)");
          data_added = TRUE;
          break;
        }
      
      if (!data_added)
      {
        dataname_text = "$(" + dataname_text + "_DATA)";
        makefile_editor_edit_target_add_element (mf, "EXTRA_DIST", dataname_text);
      }
    }
  }
  
  openldev_makefile_parser_write (mf);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_APPLY);
  gtk_widget_destroy (dialog);
}

// Add an element to the Makefile
void makefile_editor_edit_target_add_element (Makefile *mf,
                                              string name, 
                                              string attr)
{
  MakefileAttr *obj = new MakefileAttr;
  if (!attr.empty())
    obj->attr.push_back (attr);
  mf->types.push_back (MAKEFILE_ARRAY);
  mf->attributes.push_back (obj);
  mf->names.push_back (name);
} 

// Remove all of the elements for the other target
void makefile_editor_edit_target_purge_target (Makefile *mf, 
                                               string target)
{
  string base = target;
  unsigned int i, j;
  for (i=0; i < base.length(); i++)
    if (base[i] == '-' || base[i] == '.')
      base[i] = '_';
  
  string ldflags = base + "_LDFLAGS",
         ldadd = base + "_LDADD",
         libadd = base + "_LIBADD",
         sources = base + "_SOURCES";
  
  // Remove LDFLAGS
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == ldflags)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove LDADD
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == ldadd)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove LIBADD
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == libadd)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove SOURCES
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == sources)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove the target from its list
  for (i=0; i < mf->types.size(); i++)
  {
    if (mf->names[i].find ("_PROGRAMS", 0) != string::npos ||
        mf->names[i].find ("_LIBRARIES", 0) != string::npos ||
        mf->names[i].find ("_LTLIBRARIES", 0) != string::npos)
    {
      vector<string>::iterator iter = mf->attributes[i]->attr.begin();
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
      {
        if (mf->attributes[i]->attr[j] == target)
          break;
        iter++;
      }
      
      if (*iter == target)
      {
        mf->attributes[i]->attr.erase (iter);
        break;
      }
    }
  }
}

// Remove the prefix elements from the Makefile
void makefile_editor_edit_target_purge_prefix (Makefile *mf, 
                                               string prefix)
{
  unsigned int i, j;
  string dir = prefix + "dir",
         data = prefix + "_DATA",
         extradist = "$(" + prefix + "_DATA)";
  
  // Remove the directory element
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == dir)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove the data element
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == data)
    {
      makefile_editor_edit_purge_element (mf, i);
      break;
    }
  
  // Remove the EXTRA_DIST
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == "EXTRA_DIST")
    {
      vector<string>::iterator iter = mf->attributes[i]->attr.begin();
      for (j=0; j < mf->attributes[i]->attr.size(); j++)
      {
        if (mf->attributes[i]->attr[j] == extradist)
        {
          mf->attributes[i]->attr.erase (iter);
          break;
        }
        iter++;
      }
      break;
    }
}

/************************ Edit Subproject ************************/

gboolean makefile_editor_edit_subproject (MakefileEditor *editor,
                                      GtkTreeIter iter,
                                      GtkTreeModel *model)
{
  unsigned int i, j;
  string makefile;
  gchar *subproject;
  gtk_tree_model_get (model, &iter, 1, &subproject, -1);
  
  if (editor->current_dir == subproject)
    makefile = g_strconcat (editor->project->location.c_str(), 
                            editor->current_path.c_str(), "Makefile.am", NULL);
  else
    makefile = g_strconcat (editor->project->location.c_str(), 
                            editor->current_path.c_str(), 
                            subproject, "/Makefile.am", NULL);
  Makefile *mf = openldev_makefile_parser_new ((gchar*) makefile.c_str());
  
  gchar *gladefile = g_strconcat (INSTALL_PREFIX, "/share/openldev/glade/",
                                  "mfeditor-subproject.glade", NULL);
  GladeXML *xml = glade_xml_new (gladefile, NULL, NULL);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  GtkWidget *includes = glade_xml_get_widget (xml, "includes");
  GtkWidget *cflags = glade_xml_get_widget (xml, "cflags");
  GtkWidget *cxxflags = glade_xml_get_widget (xml, "cxxflags");
  
  for (i=0; i < mf->types.size(); i++)
  {
    if (mf->names[i] == "INCLUDES")
    {
      if (mf->attributes[i]->attr.size() > 0)
        gtk_entry_set_text (GTK_ENTRY (includes), mf->attributes[i]->attr[0].c_str());
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        gtk_entry_set_text (GTK_ENTRY (includes), g_strconcat (gtk_entry_get_text
                            (GTK_ENTRY (includes)), " ", 
                            mf->attributes[i]->attr[j].c_str(), NULL));
    }
    else if (mf->names[i] == "AM_CFLAGS")
    {
      if (mf->attributes[i]->attr.size() > 0)
        gtk_entry_set_text (GTK_ENTRY (cflags), mf->attributes[i]->attr[0].c_str());
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        gtk_entry_set_text (GTK_ENTRY (cflags), g_strconcat (gtk_entry_get_text
                            (GTK_ENTRY (cflags)), " ", 
                            mf->attributes[i]->attr[j].c_str(), NULL));
    }
    else if (mf->names[i] == "AM_CXXFLAGS")
    {
      if (mf->attributes[i]->attr.size() > 0)
        gtk_entry_set_text (GTK_ENTRY (cxxflags), mf->attributes[i]->attr[0].c_str());
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        gtk_entry_set_text (GTK_ENTRY (cxxflags), g_strconcat (gtk_entry_get_text
                            (GTK_ENTRY (cxxflags)), " ", 
                            mf->attributes[i]->attr[j].c_str(), NULL));
    }
  }
  
  // Run the dialog & apply the changes if the user doesn't Cancel
  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_APPLY)
  {
    gboolean applied = FALSE;
    gchar *incl = g_strdup (gtk_entry_get_text (GTK_ENTRY (includes))),
          *cflg = g_strdup (gtk_entry_get_text (GTK_ENTRY (cflags))),
          *cxfl = g_strdup (gtk_entry_get_text (GTK_ENTRY (cxxflags)));
  
    // Add the includes element
    for (i=0; i < mf->types.size(); i++)
      if (mf->names[i] == "INCLUDES")
      {
        mf->attributes[i]->attr.clear ();
        mf->attributes[i]->attr.push_back (incl);
        applied = TRUE;
        break;
      }
    
    if (!applied && !string(incl).empty())
      makefile_editor_edit_target_add_element (mf, "INCLUDES", incl);
    
    // Add the am_cflags element
    applied = FALSE;
    for (i=0; i < mf->types.size(); i++)
      if (mf->names[i] == "AM_CFLAGS")
      {
        mf->attributes[i]->attr.clear ();
        mf->attributes[i]->attr.push_back (cflg);
        applied = TRUE;
        break;
      }
    
    if (!applied && !string(cflg).empty())
      makefile_editor_edit_target_add_element (mf, "AM_CFLAGS", cflg);
    
    // Add the am_cflags element
    applied = FALSE;
    for (i=0; i < mf->types.size(); i++)
      if (mf->names[i] == "AM_CXXFLAGS")
      {
        mf->attributes[i]->attr.clear ();
        mf->attributes[i]->attr.push_back (cxfl);
        applied = TRUE;
        break;
      }
    
    if (!applied && !string(cxfl).empty())
      makefile_editor_edit_target_add_element (mf, "AM_CXXFLAGS", cxfl);
    
    openldev_makefile_parser_write (mf);
  }
    
  gtk_widget_destroy (dialog);
  g_free (gladefile);
  g_free (subproject);
  return TRUE;
}

/************************ Edit Script ************************/

// Edit a script object
gboolean makefile_editor_edit_script (MakefileEditor *editor,
                                  GtkTreeIter iter,
                                  GtkTreeModel *model)
{
  unsigned int i, j;
  int l = -1;
  gchar *script;
  gtk_tree_model_get (model, &iter, 1, &script, -1);
  
  gchar *makefile = g_strconcat (editor->project->location.c_str(), 
                                 editor->current_path.c_str(), 
                                 "Makefile.am", NULL);
  Makefile *mf = openldev_makefile_parser_new (makefile);
  
  // Load the glade file and widgets
  gchar *gladefile = g_strconcat (INSTALL_PREFIX, "/share/openldev/glade/",
                                  "mfeditor-script.glade", NULL);
  GladeXML *xml = glade_xml_new (gladefile, NULL, NULL);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  GtkWidget *header = glade_xml_get_widget (xml, "header");
  GtkWidget *content = glade_xml_get_widget (xml, "content");
  gtk_entry_set_text (GTK_ENTRY (header), script);
  
  // Get the text for the script
  for (i=0; i < mf->types.size(); i++)
    if (mf->names[i] == script)
    {
      string text;
      if (mf->attributes[i]->attr.size() > 0)
        text = mf->attributes[i]->attr[0];
      for (j=1; j < mf->attributes[i]->attr.size(); j++)
        text += "\n" + mf->attributes[i]->attr[j];
      
      // Set the text to the buffer & record the position
      GtkTextBuffer *buffer = gtk_text_buffer_new (NULL);
      gtk_text_buffer_set_text (buffer, (gchar*) text.c_str(), -1);
      gtk_text_view_set_buffer (GTK_TEXT_VIEW (content), buffer);
      l = i;
      break;
    }
  
  // Run the dialog until a button is pressed
  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_APPLY)
  {
    // Set the header of the script & reset the attributes
    GtkTextIter start, end;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (content));
    gtk_text_buffer_get_start_iter (buffer, &start);
    gtk_text_buffer_get_end_iter (buffer, &end);
    string txt = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
    mf->names[l] = gtk_entry_get_text (GTK_ENTRY (header));
    mf->attributes[l]->attr.clear();
    
    // Get all but the last line of text
    while (txt.find ("\n", 0) != string::npos)
    {
      mf->attributes[l]->attr.push_back (txt.substr (0, txt.find ("\n", 0)));
      txt.erase (0, txt.find ("\n", 0) + 1);
    }
    
    // Add the last line of text & write Makefile.am
    if (txt.length () > 0)
      mf->attributes[l]->attr.push_back (txt);
    openldev_makefile_parser_write (mf);
  }
    
  gtk_widget_destroy (dialog);
  g_free (gladefile);
  g_free (script);
  g_free (makefile);
  return TRUE;
}

/************************ Edit Variable ************************/

// Edit a variable object
gboolean makefile_editor_edit_variable (MakefileEditor *editor,
                                    GtkTreeIter iter,
                                    GtkTreeModel *model)
{
  int l = -1;
  unsigned int i;
  gchar *makefile = g_strconcat (editor->project->location.c_str(), 
                                 editor->current_path.c_str(),
                                 "Makefile.am", NULL);
  Makefile *mf = openldev_makefile_parser_new (makefile);
  
  gchar *variable;
  gtk_tree_model_get (model, &iter, 1, &variable, -1);
  
  // Find where it is in the Makefile
  for (i=0; i < mf->types.size(); i++)
    if (g_strcasecmp (mf->names[i].c_str(), variable) == 0)
      l = i;
  
  // TODO: Needs an else statement!
  if (l >= 0)
  {
    // Load the glade file & widgets
    gchar *gladefile = g_strconcat (INSTALL_PREFIX, "/share/openldev/glade/",
                                    "mfeditor-variable.glade", NULL);
    GladeXML *xml = glade_xml_new (gladefile, NULL, NULL);
    GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
    GtkWidget *var = glade_xml_get_widget (xml, "variable");
    GtkWidget *content = glade_xml_get_widget (xml, "content");
    
    // Set the information for the variable
    gtk_entry_set_text (GTK_ENTRY (var), variable);
    if (mf->attributes[l]->attr.size() > 0)
      gtk_entry_set_text (GTK_ENTRY (content), mf->attributes[l]->attr[0].c_str());
    for (i=1; i < mf->attributes[l]->attr.size(); i++)
      gtk_entry_set_text (GTK_ENTRY (content), g_strconcat (gtk_entry_get_text
                          GTK_ENTRY (content), " ", mf->attributes[l]->attr[i].c_str(), NULL));
    
    // Run the dialog & apply the changes if the user doesn't Cancel
    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_APPLY)
    {
      mf->names[l] = g_strdup (gtk_entry_get_text (GTK_ENTRY (var)));
      mf->attributes[l]->attr.clear();
      mf->attributes[l]->attr.push_back (gtk_entry_get_text (GTK_ENTRY (content)));
      openldev_makefile_parser_write (mf);
    }
    
    gtk_widget_destroy (dialog);
    g_free (gladefile);
  }
  
  g_free (variable);
  g_free (makefile);
  return TRUE;
}
